# Jouons avec la DB de Noethys

## Inspection DB de Noethys

Pour utiliser Django avec une db propre + une db étrangère, on peut s'appuyer sur https://docs.djangoproject.com/en/2.0/topics/db/multi-db/

J'ai créé un projet toy avec une app chticonn.

Dans `toy/settings/base.py` :
```
DATABASES = {
      'default': env.db_url(
          'DJANGO_DATABASE_URL', default='sqlite:///{}'.format(
        base_dir('sqlite.db'))),
      'noethys': {
        'ENGINE': 'django.db.backends.sqlite3',
        .         'NAME': '../Noethys/noethys/Static/Exemples/EXEMPLE_cantine_DATA.dat',
      },
}
```

Puis on inspecte la DB avec :
```
./manage.py inspectdb --database noethys > chticonn/models.py
```

Remarques à froid:

- ça fonctionne bien
- il n'y a pas de primarykey
- il n'y a pas de foreign keys
- donc pas de politique on_delete

**=> ça va être à retravailler à la main**

## Utilisation de la DB étrangère dans django

Un coup de
```
./manage.py makemigrations --dry-run
```
montre que django veut créer les tables. Normal : pour le moment il travaille avec la DB default; il faut qu'on branche la bd noethys sur les modèles de l'appli chticonn.

La doc: https://docs.djangoproject.com/en/2.0/topics/db/multi-db/#automatic-database-routing

On défini dans `chticonn/routers.py`
```
class NoethysRouter:
    def db_for_read(self, model, **hints):
        if model._meta.app_label == 'chticonn':
            return 'noethys'
        return None

    def db_for_write(self, model, **hints):
        if model._meta.app_label == 'chticonn':
            return 'noethys'
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        We allow cross db relations.
        FIXME: Note it might not works outside mysql/sqlite relations
        """
        return True

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        if model._meta.app_label == 'chticonn':
            return False
        return True
```
ainsi que
```
DATABASE_ROUTERS = ['chticonn.routers.NoethysRouter']
```
dans `toy/settings/base.py`

Ensuite on crée et applique les migrations. Comme c'est `managed = false`, il n'y aura pas de creation/destruction/altération de table.

On peut ensuite tester:
```
./manage.py shell
from chticonn import models
models.Factures.objects.all()
```

Et ça pas marche. Normal: il faut définir une `primary key`.

On défini dans `chticonn/models.py`
```
idfacture = models.IntegerField(db_column='IDfacture', primary_key=True)
```
Et alors
```
In [3]: models.Factures.objects.all()
Out[3]: <class 'chticonn.models.Factures'>
<QuerySet []>
```
Et de la même façon
```
In [2]: models.Individus.objects.all()
Out[2]: <class 'chticonn.models.Individus'>
<QuerySet [<Individus: Individus object (1)>, <Individus: Individus object (2)>, <Individus: Individus object (3)>, <Individus: Individus object (4)>, <Individus: Individus object (5)>, <Individus: Individus object (6)>, <Individus: Individus object (7)>, <Individus: Individus object (8)>, <Individus: Individus object (9)>, <Individus: Individus object (10)>, <Individus: Individus object (11)>, <Individus: Individus object (12)>, <Individus: Individus object (13)>, <Individus: Individus object (14)>, <Individus: Individus object (15)>, <Individus: Individus object (16)>, <Individus: Individus object (17)>, <Individus: Individus object (18)>, <Individus: Individus object (19)>, <Individus: Individus object (20)>, '...(remaining elements truncated)...']>
```

## Altération au runtime du modèle (échec)

Il s'agit de faire quelque chose genre:
```
from django.db import models
from .inspected_models import *

from django.db.models.signals import class_prepared

def add_primary_key (sender, **kwargs):
    if sender.__name__ == "Individus":
        sender.add_to_class('bling', 'blang')

class_prepared.connect(add_primary_key)

Individus._prepare()
```

**Problème : ça ne fonctionne pas avec les primary keys**

Solution: ne pas faire ça. Plutôt hacker inspectdb.

## Hackons plutôt inspectdb

Malheureusement, après avoir essayé de jouer avec les correspondances tables/colonnes le seul indice fiable à prendre en compte est la position de la colonne. Donc on part la dessus.

Le code est dans `chticonn/management/commands/inspectdb.py`

Au final, ça fonctionne fonctionne mais nécessite un fork quasi complet de inspectdb.

```
./manage.py inspectdb --database noethys > chticonn/models.py
./manage.py shell --command 'from chticonn import models; print([models.Individus.objects.all()]);'
<class 'chticonn.models.Individus'>
[<QuerySet [<Individus: AUTRET>, <Individus: AUTRET>, <Individus: AUTRET>, '...(remaining elements truncated)...']>]
```

Relation traversantes :
```
In [1]: from chticonn import models

In [2]: set([c.individu for c in models.Consommations.objects.all()])
Out[2]:
{<Individus: ALLIBERT>,
 <Individus: AUTRET>,
 <Individus: CLOAREC>,
 <Individus: HAMON>}

In [3]: [u.consommations_set.all() for u in models.Utilisateurs.objects.all()]
Out[3]: [<QuerySet [<Consommations: Consommations object (1)>, <Consommations: Consommations object (2)>, '...(remaining elements truncated)...']>]
```

### Notes

- les conventions de tables/colonnes sont mouvantes ; le seul indice fiable pour
  savoir qu'on a une primary key c'est la position de la colonne dans la table.
- le modèle étant non-managé, dumpdata ne fonctionne pas et il va falloir hacker
  un peu pour les tests. Cf https://code.djangoproject.com/ticket/13816 et
  https://dev.to/patrnk/testing-against-unmanaged-models-in-django
- il y a des différences d'introspection entre mysql et sqlite, mais
  actuellement ça fonctionne sur les deux. Probablement il faudra tester sur les
  deux, sinon en priorité sur mysql (et/ou pgsql).
  **(C'était un bug. Cf https://forge.cliss21.org/fpoulain/resthys/commit/a12859102)**
- il y a une table de paramètres. Par ex:

```
In [1]: from chticonn import models

In [2]: models.Parametres.objects.get(nom="version").parametre
Out[2]: '1.2.3.3'

```
- django crééra tôt ou tard une table django_migrations qui ne respecte pas les
  regles du routeur et n'est pas optionnelle. Cf django/db/migrations/recorder.py
  et https://code.djangoproject.com/ticket/23273

## TODO:

- écrire des sérialiseurs dédiés pour gérer les cochoncetés du type champ liste csv

**proprification future :**

- sortir les tables d'adaptation du code, les mettre dans de la conf
- affiner le choix du champ str; par ex avec une table de lambda

### Done:

- vérifier que check n'a pas de warning et que dumpdata fonctionne
- primary keys créées
- ajouter la méthode str (prend le champ nom)
- reconnaitre les foreign keys
- définir les on delete
- virer les prefixes id dans les modeles django, et utiliser id pour les primary keys
- gérer les conflits. En particulier on a :
- tests fonctionnels

```
chticonn.Prestations.idfacture: (fields.E303) Reverse query name for 'Prestations.idfacture' clashes with field name 'Factures.prestations'.
  HINT: Rename field 'Factures.prestations', or add/change a related_name argument to the definition for field 'Prestations.idfacture'.
```

## Mise en place de tests fonctionnels

Le framework de test de Django tolère assez mal une DB étrangère, car Django
tente de base de substituer la DB en cours par une DB de test créée et migrée
pour l'occasion. Or nos modèles étrangers sont non managés donc ne migrent pas.

Pour s'affranchir des problèmes posés, il suffit d'utiliser une base de données
déjà existante et de forcer l'option keepdb. Les autres solutions testées (cf
plus haut) n'ont pas été concluantes. Pour ça j'ai surchargé la commande `test`,
dans `chticonn/management/commands/test.py`.

On teste:
* l'adéquation entre le modèle chaargé et le modèle inspecté ;
* quelques vérifications de base sur les modèles ;
* quelques manipulations de base sur et entre quelques modèles.

## Rétro ingénérie d'un paiement Noethys

### Génération des factures

La génération des factures:
* rempli la table `factures` ;
* référence les prestations concernées avec le numéro de facture.

Les champs `regle` et `solde` de la facture sont visiblement à l'abandon après
création. Noethys se base en pratique sur les reglements pour la présentation du
solde (cf `noethys/Utils/UTILS_Facturation.py`, lignes 413 et 496). On peut
imaginer que c'est pour faciliter la gestion des impayés.

### Paiment de facture

Un paiement :
* est inscrit à l'historique:
* inscrit un règlement :
* inscrit des ventilations (1 par prestation associée au règlement, donc à la
  facture dans le cas qui nous concerne):

Éventuellement un nouveau payeur est créé à l'occasion du paiement.

**Note :** il faudra aussi gérer le type de paiement
