import re
from collections import OrderedDict
from django.db import connections

from django.core.management.commands.inspectdb import (
    Command as InspectDBCommand,
)

class Command(InspectDBCommand):
    def handle(self, **options):
        if options.get('database') != 'noethys':
            self.handle_inspection= super().handle_inspection
        try:
            for line in self.handle_inspection(options):
                self.stdout.write("%s\n" % line)
        except NotImplementedError:
            raise CommandError("Database inspection isn't supported for the currently selected database backend.")

    def normalize_col_name(self, table_name, col_name, used_column_names, is_relation, is_primary):
        new_name, field_params, field_notes= super().normalize_col_name (col_name, used_column_names, is_relation)
        in_conflicts= {
                ('factures', 'prestations'): 'prestations_list',
                }
        if is_primary:
            new_name= 'id'
        elif is_relation and new_name.startswith('id') and len(new_name) > 2:
            new_name= new_name[2:]
        elif (table_name, col_name) in in_conflicts:
            new_name= in_conflicts[(table_name, col_name)]
            field_params['db_column']= col_name

        return new_name, field_params, field_notes

    def handle_inspection(self, options):
        # on delete matrix for foreign key
        on_delete= {
        #       ('consommations', 'IDindividu'): 'CASCADE',
                }
        connection = connections[options['database']]
        # 'table_name_filter' is a stealth option
        table_name_filter = options.get('table_name_filter')

        def table2model(table_name):
            return re.sub(r'[^a-zA-Z0-9]', '', table_name.title())

        def strip_prefix(s):
            return s[1:] if s.startswith("u'") else s

        def guess_relations (tables, connection):
            relations= {}
            for table_name in tables:
                try:
                    table_description = connection.introspection.get_table_description(cursor, table_name)
                    primary_key_column = table_description[0].name
                    relations[primary_key_column] = (primary_key_column, table_name)
                except Exception as e:
                    continue
            return relations

        with connection.cursor() as cursor:
            yield "# This is an auto-generated Django model module."
            yield 'from %s import models' % self.db_module
            known_models = []
            tables_to_introspect = options['table'] or connection.introspection.table_names(cursor)
            relations = guess_relations (tables_to_introspect, connection)

            for table_name in tables_to_introspect:
                if table_name_filter is not None and callable(table_name_filter):
                    if not table_name_filter(table_name):
                        continue
                try:
                    table_description = connection.introspection.get_table_description(cursor, table_name)
                    primary_key_column = table_description[0].name
                except Exception as e:
                    yield "# Unable to inspect table '%s'" % table_name
                    yield "# The error was: %s" % e
                    continue

                yield ''
                yield ''
                yield 'class %s(models.Model):' % table2model(table_name)
                known_models.append(table2model(table_name))
                used_column_names = []  # Holds column names used in the table so far
                column_to_field_name = {}  # Maps column names to names of model fields
                for row in table_description:
                    comment_notes = []  # Holds Field notes, to be displayed in a Python comment.
                    extra_params = OrderedDict()  # Holds Field parameters such as 'db_column'.
                    column_name = row[0]
                    is_relation = column_name in relations
                    is_primary = column_name == primary_key_column

                    att_name, params, notes = self.normalize_col_name(table_name,
                        column_name, used_column_names, is_relation, is_primary)
                    extra_params.update(params)
                    comment_notes.extend(notes)

                    used_column_names.append(att_name)
                    column_to_field_name[column_name] = att_name

                    if is_primary:
                        extra_params['primary_key'] = True

                    if is_relation and column_name != primary_key_column:
                        rel_to = (
                            "self" if relations[column_name][1] == table_name
                            else table2model(relations[column_name][1])
                        )
                        if rel_to in known_models:
                            field_type = 'ForeignKey(%s' % rel_to
                        else:
                            field_type = "ForeignKey('%s'" % rel_to
                    else:
                        # Calling `get_field_type` to get the field type string and any
                        # additional parameters and notes.
                        field_type, field_params, field_notes = self.get_field_type(connection, table_name, row)
                        extra_params.update(field_params)
                        comment_notes.extend(field_notes)

                        field_type += '('

                    # Don't output 'id = meta.AutoField(primary_key=True)', because
                    # that's assumed if it doesn't exist.
                    if att_name == 'id' and extra_params == {'primary_key': True}:
                        if field_type == 'AutoField(':
                            continue
                        elif field_type == 'IntegerField(' and not connection.features.can_introspect_autofield:
                            comment_notes.append('AutoField?')

                    # Add 'null' and 'blank', if the 'null_ok' flag was present in the
                    # table description.
                    if row[6]:  # If it's NULL...
                        if field_type == 'BooleanField(':
                            field_type = 'NullBooleanField('
                        elif 'primary_key' not in extra_params or extra_params['primary_key'] != True:
                            extra_params['blank'] = True
                            extra_params['null'] = True

                    field_desc = '%s = %s%s' % (
                        att_name,
                        # Custom fields will have a dotted path
                        '' if '.' in field_type else 'models.',
                        field_type,
                    )
                    if field_type.startswith('ForeignKey('):
                        if (table_name, column_name) in on_delete:
                            field_desc += ', on_delete=models.{}'.format(on_delete[(table_name, column_name)])
                        else:
                            field_desc += ', on_delete=models.SET_NULL'

                    if extra_params:
                        if not field_desc.endswith('('):
                            field_desc += ', '
                        field_desc += ', '.join(
                            '%s=%s' % (k, strip_prefix(repr(v)))
                            for k, v in extra_params.items())
                    field_desc += ')'
                    if comment_notes:
                        field_desc += '  # ' + ' '.join(comment_notes)
                    yield '    %s' % field_desc
                if 'nom' in used_column_names:
                    yield '    def __str__(self):'
                    yield '        return self.nom'
                for meta_line in self.get_meta(table_name, {}, column_to_field_name):
                    yield meta_line

