from django.test import TestCase
from django.core.management import call_command
from django.db import connections
import io, inspect

from chticonn import models

unique_name="PoluX_Boop6Eingiez"

class InspectDBTestCase(TestCase):
    def test_coherence (self):
        buf= io.StringIO()
        rc= call_command('inspectdb', '--database=noethys',
                stdout=buf)
        self.assertIsNone(rc, msg="""

        This command failed:
        ./manage.py inspectdb --database=noethys

        """)
        buf.seek(0)
        inspected_db= buf.read()
        buf.close()
        with open('chticonn/models.py', 'r') as f:
            self.assertEqual(f.read(), inspected_db, msg="""

            Your chticonn/models.py doesn't seems to be coherent with the
            inspected database. You have to refresh it using
            ./manage.py inspectdb --database=noethys > chticonn/models.py
            and them migrate and run migrations.

            """)

class modelsLoadedTestCase(TestCase):
    def setUp(self):
        self.loaded_models= [m for _,m in models.__dict__.items()
                if inspect.isclass(m) and issubclass(m, models.models.Model)]

    def test_primary_key_is_id (self):
        [self.assertEqual(m._meta.pk.name, 'id') for m in self.loaded_models]

    def test_foreign_key_do_not_startwith_id (self):
        # The inverse not has to be true: and id.* field might refers to
        # an hardcoded choice value
        for m in self.loaded_models:
            for r in m._meta.related_objects:
                self.assertFalse(r.name.startswith('id'))

class IndividusTestCase(TestCase):
    def setUp(self):
        models.Activites.objects.filter(nom=unique_name).delete()
        models.Consommations.objects.filter(individu__nom=unique_name).delete()
        models.Individus.objects.filter(nom=unique_name).delete()

        i= models.Individus.objects.create(nom=unique_name)
        a= models.Activites.objects.create(nom=unique_name)
        c= models.Consommations.objects.create(individu=i, activite=a)

    def test_created(self):
        i = models.Individus.objects.get(nom=unique_name)
        self.assertTrue(i.id)
        self.assertEqual(i.nom, unique_name)
        c = models.Consommations.objects.get(individu=i)
        self.assertTrue(c.id)
        self.assertEqual(c.activite.nom, unique_name)

    def test_individu_to_consommation_request (self):
        # get consommations from individu name
        r= models.Consommations.objects.filter(individu__nom=unique_name)
        self.assertEqual(len(r), 1)
        self.assertEqual(r[0].individu.nom, unique_name)
        self.assertEqual(r[0].activite.nom, unique_name)

    def test_activite_to_consommation_request (self):
        # get consommations from activite name
        r= models.Consommations.objects.filter(activite__nom=unique_name)
        self.assertEqual(len(r), 1)
        self.assertEqual(r[0].individu.nom, unique_name)
        self.assertEqual(r[0].activite.nom, unique_name)

    def test_activite_to_individu_request (self):
        # get all individu having a consommation with given activite name
        activites= models.Activites.objects.filter(nom=unique_name)
        consos= models.Consommations.objects.filter(activite__in=activites)
        r= models.Individus.objects.filter(consommations__in=consos)
        self.assertEqual(len(r), 1)
        self.assertEqual(r[0].nom, unique_name)

    def test_individu_deletion(self):
        i = models.Individus.objects.get(nom=unique_name)
        self.assertEqual(i.delete()[0], 1)
        self.assertEqual(len(models.Individus.objects.filter(nom=unique_name)), 0)

    def test_consommation_deletion(self):
        c = models.Consommations.objects.get(activite__nom=unique_name)
        self.assertEqual(c.delete()[0], 1)
        self.assertEqual(len(models.Consommations.objects.filter(activite__nom=unique_name)), 0)

    def test_activite_deletion(self):
        a = models.Activites.objects.get(nom=unique_name)
        self.assertEqual(a.delete()[0], 1)
        self.assertEqual(len(models.Activites.objects.filter(nom=unique_name)), 0)

    def tearDown(self):
        models.Activites.objects.filter(nom=unique_name).delete()
        models.Consommations.objects.filter(individu__nom=unique_name).delete()
        models.Individus.objects.filter(nom=unique_name).delete()
